/*

We introduced the idea of teams but it isn't complete. Both
blue mobs are on the same team, but you could beat up one
while the other stands by and watches. They're not very good
teammates!

To address this we add a new proc for ai mobs: call_for_help()
Look at the code to see how this proc works and how it is used.


  -- When you run this example --

When you run this example, attack one blue mob and run towards
the other blue mob (but don't attack it). When the blue mob you
attacked first catches up it will call for help and the other
blue mob will attack you too (even though you never attacked it
directly).

*/

var
	const
		TEAM_1 = 1
		TEAM_2 = 2
		TEAM_3 = 3

mob
	Click()
		if(get_dist(src, usr) <= 1)
			usr.attack(src)

	Stat()
		stat(src)
		stat("team", team)

		for(var/mob/ai/m in world)
			stat(" ")
			stat("AI mob", m)
			stat("team", m.team)
			stat("target", m.target)

mob

	var
		team = 0
	proc
		ai()
		hit_by(mob/m)
		attack(mob/target)
			show_hit(target)
			target.hit_by(src)

	Login()
		. = ..()
		team = TEAM_1
		icon_state = "green"

	ai
		var
			mob/target

		New()
			..()
			spawn(rand(10,20)) ai()

		ai()
			if(target)
				// If the mob has a target we want them to always call for help.
				call_for_help()
				if(get_dist(src, target) > 1)
					step_to(src, target)
					sleep(5)
				else
					attack(target)
					sleep(10)
			else
				get_target()
				if(target)
					sleep(5)
				else
					step_rand(src)
					sleep(20)

			spawn() ai()

		hit_by(mob/m)
			if(m.team == team) return

			if(!target)
				target = m

		proc
			get_target()

			// We define call_for_help here so that all children of the
			// /mob/ai type can use call_for_help. The proc is empty
			// because, by default, it does nothing. Child types of AI
			// mobs can define their own effects.
			call_for_help()

		red
			team = TEAM_2
			icon_state = "red"

			get_target()
				for(var/mob/m in oview(2, src))
					if(m.team != team)
						target = m
						return

			// We don't define call_for_help for red mobs. Red mobs
			// are tough and don't want anyone else to help them. The
			// proc still gets called because red and blue mobs share
			// the same ai proc, their call for help just does nothing.

		blue
			team = TEAM_2
			icon_state = "blue"

			// We define call_for_help for blue mobs to make all of its
			// AI-controlled teammates within 5 tiles attack its target,
			// provided they didn't already have a target.
			call_for_help()
				for(var/mob/ai/m in oview(5, src))
					if(m.team == team)
						if(!m.target)
							m.target = target