/*

 *** First Intermission ***

We've covered a lot so far and it isn't going to get easier. Before going further
you might want to review what's been covered up to this point.


Summary

Here is a summary of the incremental changes we've made:

code-01.dm:
  Monsters wander randomly.

code-02.dm:
  Monsters wander randomly but will begin to fight back if you attack them

code-03.dm:
  Red mobs are aggressive and will attack you if you get near them.

code-04.dm:
  We added teams. Mobs won't attack someone on their own team.

code-05.dm:
  Added call_for_help. Red mobs think they're tough and don't need help. Blue
  mobs will call for help: all of their teammates within 5 tiles will join the fight.

code-06.dm:
  Mobs prefer different types of targets and will attack their most-preffered target.
  Red mobs attack the most powerful enemy, blue mobs attack the one with the lowest health.


Review -- How it Works

All AI-controlled mobs are children of the /mob/ai type. This base type defines
the basic AI thought process (the ai proc) for our red and blue mobs. The ai proc
calls helper procs (get_target, call_for_help). To change the behavior of a mob we
can use the same ai proc and change the behavior of the helper procs.

A good example of how we used this is the get_target proc. Red mobs are aggressive
and will attack you before you attack them. Blue mobs are passive and won't attack
you unless they are given a reason to (either you attack them or a teammate of theirs
calls for help). To achieve this, all we had to do was define the get_target proc
for red mobs and leave it blank for blue mobs.


Exercises

Here are some exercises you might want to try to see how well you understand what's been
covered. I suggest using code-06.dm as a starting point and add code to implement what each
exercise describes.

1-1: Crazy

We don't need to use the same ai proc for all children of /mob/ai. Create a new
type of mob called /mob/ai/crazy which wanders randomly at a faster pace and never
attacks anyone. (An answer is available in e1-1.dm)

1-2: Chicken

Create a new type of mob, /mob/ai/chicken. The chicken mob should only acquire a target
when it is attacked. When the chicken mob has a target it always runs away from it (hint:
use step_away). (An answer is available in e1-2.dm)

1-3: Crier

Create a new type of mob, /mob/ai/crier, that behaves like the chicken mob from the
previous exercise. The crier runs from its target but it also calls for help, alerting
its allies of your presence. (An answer is available in e1-3.dm)


Discussion: State Machines

You might have heard the term "finite state machine" or just "state machine". The term is
often used on the BYOND forums and resources when talking about enemy AI. It is important
to understand what a finite state machine is so that you know how it can help you to create
better enemy AI.

So far we've created pretty good AI for our mobs. We've seen some interesting behavior
emerge from making simple, incremental changes, and we've done all of this without using
a state machine. Do we need one?

Currently the mobs just react to things that they observe and make decisions based on
some variables. For example, red mobs will target any enemy that comes near them. This
is a reaction to something they observe. Red mobs also make decisions based on the power
variable: they attack the enemy with the highest power.

A state machine introduces a new variable (called the state) which we can use to help the
mob make decisions. The state variable is often necessary because it models something that
is not represented by any existing variable or local condition. For now, think of the mob's
state as its mood.

The state variable (the mob's mood) can have a finite number of values. For example, let's
use only the two values HAPPY and ANGRY. The only way we can determine if a mob is happy or
angry is by the state variable. We can check its health variable to see if it is weak. We can
check its power variable to see if it is strong, but we have no way (aside from the state) to
know if it is happy or angry.

We might use the state to change the red mob's behavior. Currently, red mobs will attack
any enemy that comes near them. Instead, here's how we want them to behave:

    If the red mob is happy:
      It only attacks enemies that attacked it first
    If the red mob is angry:
      It attacks any enemy that comes near it

This shows how we can use the state variable to achieve a more complicated behavior. There
was no easy way to achieve this behavior without the state variable.

But, we're not finished. We've defined how the mob behaves for each state it could be in, but
we haven't defined how it transitions from one state to another. We'll use this rule:

    Red mobs start as being angry.
    When a red mob kills its enemy, it becomes happy.

To implement this we first need some new features. As of code-06.dm, mobs can attack each
other but do not actually take damage so they never die. code-08a.dm adds damage. code-08b.dm
adds the state machine to modify the red mob's behavior as we just described.

*/