/*

In this file we add the state machine to modify the behavior of red
mobs as follows:

    If the red mob is happy:
      It only attacks enemies that attacked it first
    If the red mob is angry:
      It attacks any enemy that comes near it

    Red mobs start as being angry.
    When a red mob kills its enemy, it becomes happy.

You may be surprised at how simple this change is.

Note: In earlier examples, all comments in the code were about new
changes. Because the code is getting more complicated I started leaving
some old comments in there.

*/

var
	const
		TEAM_1 = 1
		TEAM_2 = 2
		TEAM_3 = 3

		ANGRY = "angry"
		HAPPY = "happy"

mob
	Click()
		if(get_dist(src, usr) <= 1)
			usr.attack(src)

	// We add the state display for each mob.
	Stat()
		stat(src)
		stat("team", team)
		stat("health", health)

		for(var/mob/ai/m in world)
			stat(" ")
			stat("AI mob", m)
			stat("team", m.team)
			stat("health", m.health)
			stat("state", m.state)
			stat("target", m.target)

mob

	var
		team = 0
		power = 0
		health = 0

	proc
		ai()
		hit_by(mob/m)
		attack(mob/target)
			if(target.dead) return

			show_hit(target)
			target.hit_by(src)

			target.health -= 1
			if(target.health <= 0)
				target.die()
				killed(target)

		// When a mob kills another, this event is called (src is the mob
		// that got the kill, m is the mob they killed). The default action
		// for this event is to do nothing. In this example we define a new
		// behavior for red mobs when they get a kill.
		killed(mob/m)
		die()
			if(!dead)
				world << "[name] dies."
				dead = 1
				icon_state = "[icon_state] dead"

	Login()
		. = ..()

		power = 20
		health = 20

		team = TEAM_1
		icon_state = "green"

	ai
		var
			mob/target

			// We add the state variable for all children of /mob/ai, even
			// though we only use it for red mobs in this example.
			state = HAPPY

		New()
			..()
			spawn(rand(10,20)) ai()

		ai()
			if(dead) return

			get_target()
			if(target)
				call_for_help()

				if(get_dist(src, target) > 1)
					step_to(src, target)
					sleep(5)
				else
					attack(target)
					sleep(10)
			else
				step_rand(src)
				sleep(20)

			spawn() ai()

		hit_by(mob/m)
			change_target(m)

		proc
			get_target()
			call_for_help()

			// The default behavior is to accept the new target
			// only if you didn't already have one.
			change_target(mob/m)
				if(!m) return
				if(m.team == team) return
				if(target) return

				target = m

		red
			power = 30
			health = 40
			team = TEAM_2
			icon_state = "red"

			// Red mobs start as being angry
			state = ANGRY

			// When a red mob kills another mob it becomes happy.
			// Note: for the killed proc to be called the red mob
			// has to be the one that gets the kill. If another mob
			// kills the red mob's target, the red mob will stay angry.
			killed(mob/m)
				state = HAPPY

			get_target()
				// If we have a target and its dead, lose that target.
				if(target && target.dead)
					target = null

				// Red mobs are only aggressive when they are angry.
				if(state == ANGRY)
					for(var/mob/m in oview(2, src))
						change_target(m)

			// red mobs target the enemy with the highest power.
			change_target(mob/m)
				if(!m) return
				if(m.team == team) return

				// ignore dead targets
				if(m.dead) return

				if(!target)
					target = m
					return

				if(m.power > target.power)
					target = m

		blue
			power = 10
			health = 5

			team = TEAM_1
			icon_state = "blue"

			get_target()
				// blue mobs don't look for targets, but if they have a
				// target and it dies we want them to lose that target.
				if(target && target.dead)
					target = null

			call_for_help()
				for(var/mob/ai/m in oview(5, src))
					if(m.team == team)
						m.change_target(target)

			// blue mobs target the enemy with the lowest health
			change_target(mob/m)
				if(!m) return
				if(m.team == team) return

				// ignore dead targets
				if(m.dead) return

				if(!target)
					target = m
					return

				if(m.health < target.health)
					target = m
