/*

 *** Second Intermission ***

The code has gotten a lot more complicated in these last few examples. The more
complicated it gets the more important it is that you understand the details and
how to properly set up your code. If the AI gets too complex you'll have a very
tough time fixing bugs and adding new behaviors.

Summary

Here is a summary of the incremental changes we've made since the last intermission:

code-08a.dm:
  Added damage and dying to set things up for the next example.

code-08b.dm:
  Added a simple state machine so the red mob can either be HAPPY or ANGRY. ANGRY
  red mobs attack any enemy that approaches. HAPPY red mobs only attack mobs that
  attack them first.

code-09.dm:
  Added a state machine so that blue mobs have a 50% chance to run away when they
  get weak.

code-09b.dm
  Showed some alternative ways to create a looping ai proc and the problems (most
  notably the call stack problem) that some methods might have.

code-10.dm:
  Showed some important ways to organize your code to avoid repeating sections
  of the same code. When you add new states you might end up repeating a lot of
  code so this is very important.


Review -- State Machines

Using a state machine you can create some interesting behavior, even if the
behavior for any single state is still simple.

A state variable can be introduced to represent something that no current variable
represents. The state is used to represent the red mob's mood. We also used the state
to represent a decision made by the blue mob. For both of these mobs we clearly defined
which states exist, what transitions exist, and what causes the transitions.


Review -- Code Organization

If your state machine has 5 states and you add another, you could add up to 10 new
transitions (5 being from each existing state to the new one and another 5 from the
new state back to each existing one). The size of the code in the ai proc can grow
very quickly.

There are two important things you should do:

1. Keep a formal descrption of your state machine (what states exist, what transitions
exist, and what causes each transition). For each state write a short description of
the mob's behavior in that state. This will come in handy when you want to add a new
behavior because you can see, without having to dig through the code, what states you
will need to add and what transitions you need to add.

This can also help in debugging. You can compare the state the mob is in to the state
you expect they would be in, then see what transitions must have happened to end up
in the wrong state. From there, you can figure out why that transition occurred when it
wasn't supposed to.

2. Keep the number of states as small as possible. Adding a new state can increase the
complexity by a lot. If you don't need to add a new state, don't! All of the examples
before code-07.dm didn't use state machines and we had some pretty neat behavior. The
easiest way to add a new behavior is often the one that requires the fewest states and
transitions.

One way to keep the number of states down while still introducing some variety in the
mobs behavior is to add a new, unchanging variable. It could be set randomly when the
mob is created. For example, you might add a variable called "evil". An evil mob (if
evil is set to 1) might kill an ally and take their money if the ally is weak and nobody
else is nearby. Mobs never transition from being evil to being not evil so this
doesn't affect your state machine, but it's still a way you can have mobs with different
behaviors.


Other Thoughts -- Behavior-Driven Development

Keep the emphasis on what behavior your mobs are capable of performing. This is what
should drive all changes that you make. Each incremental change in these examples was
either the addition of a behavior or the addition of some basic features (teams, damage,
etc.) to allow for the addition of new behaviors.

When you want to add a new feature to the game, think of how it will impact the AI.
Would the new feature require you to change any existing behaviors? Would it require
you to add any new behaviors?


Other Thoughts -- Multiple State Variables

My basic definition of a state machine is the introduction of a new variable that can
be used to help the mob make decisions. Any variable you use to make decisions is
part of the mob's state. Consider this example:

In a capture the flag game each player might have a "mood" type of state: aggressive,
cautious, sneaky, defensive. This isn't the only variable that is important in deciding
the mob's actions. You also need to consider if the mob is carrying the flag.

When you have the flag all actions should, in some way, advance the flag back towards
your own base. If you don't have the flag this is not important.


Exercises

Here are some ideas of things you could add. It might not be necessary to implement them,
but you should really think about how you would implement them.

2-1: Ranged Attacks

Currently all mobs must be next to their target in order to attack them. How would you
create a mob with a ranged attack? If the mob could attack its target from 4 tiles away
it would want to balance attacking with moving away from the target (to keep its distance).
Would you need a state machine for this? If so, what states would you need? What do the
states represent?

2-2 Multiple Targets

Even in the last example the targeting method is rather simple. Mobs only have a current
target. If a mob finds a better target it forgets about its first target. What changes
would you need to make so that a mob would remember its previous targets? To test this,
have a mob switch targets then kill its new target. Does it go back to the first target?
Always? What if the first target ran away?

2-3 Non-Offensive Abilities

In all of these examples the mobs only move and attack (running away is just moving in
a different direction). What if mobs had an ability to heal an ally? How would you
determine when the mob should heal an ally or when they should attack an enemy? Would you
use states for this? If so, why?

(code-12.dm explains techniques that can be used to implement this, and the examples
in code-13.dm and code-14.dm show how those techniques would be used specifically to
implement a mob that can either attack or heal an ally.)

*/