/*

 *** The End ***

Summary

Here is a summary of the changes we've made since the last intermission:

code-12.dm:
  Introduced the concepts of utility, heuristics, and greedy strategies. We used
  these concepts to make the mob move towards its destination,

code-13.dm:
  Applied the concept of utility to make blue mobs decide when to attack their
  enemy and when to heal an ally.

code-14.dm:
  Added a state machine to the previous example. The state represents the action
  that the blue mob has decided to take. To carry out the action the mob might
  need to move closer to its target. Whenever the action is complete the mob
  goes back to the original state to decide what action should be taken next.

code-15.dm
  Used invisible markers on the map to create mobs that hide in good hiding
  places when they get weak.

code-16.dm
  Changed how delays in the ai proc work to make mobs react faster.

code-17.dm
  Made mobs that chase after their target in the way a human would. If their
  target runs out of their view, the mob continues running to the last location
  their target was seen at.


Review -- Utility

Utility is a way of assigning a number that represents how good a situation is.
If we know what actions a mob can perform and estimate how they would affect the
current situation, we can figure out what situation we can reach that has the
greatest utility.

It might not be clear precisely how the function should be set up, but it should
be clear what variables should affect the value of the function.


Review -- Heuristic

Sometimes the utility of a situation can be hard to calculate. Instead of using
a time-consuming process to estimate utility we can use a simpler rule, called
a heuristic.

In code-12.dm we used a heuristic to estimate how far a mob is from a destination.
The value we're trying to estimate is the number of steps, taking obstacles into
account, that it would take the mob to reach its destination. We can't compute
the actual value unless we compute the entire path to the destination, but we're
trying to avoid that because it's more time consuming. Instead, we just use
get_dist to estimate the number of steps.


Review -- Greedy Strategies

The mob in code-12.dm is said to be greedy because it picks the single action
that appears to be the best choice. In this case, that's the action that yields
the most utility. This isn't necessarily a bad thing. Picking the action that
gives the least utility would be a bad thing. But, being greedy can be bad
because the mob isn't looking ahead. It only considers single actions and can
get stuck in a non-optimal location if every action from that location reduces
the utility (even though a sequence of moves exists that would increase the
utility).


Other Thoughts -- How To Use AI in Games

For certain types of games you might think you don't have a need for AI. For
example, you might be creating a game that will only feature human-controlled
players. Do you still need AI? If so, how can you use it?

  -- Using AI with Human-Controlled Players --

We've been using AI to make computer-controlled mobs perform actions, but you
can use all of these techniques to assist human-controlled mobs too. Here are
a few examples:

1. Simplify movement by letting the player click on a tile and use AI to move
the player to that location.

2. Use AI routines to make suggestions for how the player should act. In a
strategy game you could run an AI routine to figure out what a computer-
controlled player would do in the human player's current situation, then
suggest that action to the human as a way of helping them.

3. Make abilities more intelligent. Instead of telling a player that they are
not within range of their target to use the selected ability, move them towards
the target then use the ability. This is exactly what we did in code-14.dm,
except now we're applying it to a human-controlled mob.


Other Thoughts -- Concepts vs. Techniques vs. Behaviors

A state machine is a conceptual thing. State machines don't have to be used
for AI (in fact, their biggest use is *not* in AI). A technique is a method
of taking a concept, like state machines, and applying it to AI. A behavior
is an action (or pattern of action) that you can create by using techniques.

In these examples I've covered all three of these topics. By understanding
the concepts and techniques you should be able to create new behaviors that
weren't seen in any of these examples.


Exercises

After going through all of the examples, here are some things you might want
to try:

3-1 Different Type of Game

All of the examples were for top-down action games. Try converting some
examples (or creating your own example from scratch) to work in a turn-
based game, or a side-scrolling game. What things do you need to worry
about when you implement AI in a different type of game? What things don't
you need to worry about?

3-2 Incomplete Knowledge

In code-17.dm (mobs that chase you) we modeled the mob's incomplete knowledge
of the world by having them remember the last location where they saw you.
Can you think of any other situations where you could use a similar technique?

3-3 Invisible Markers

In code-15.dm we placed invisible markers on the map that tell mobs where
the good hiding places are. Can you think of a way you could use this
technique in a game of yours?

The markers in the example contained no information. We only used the markers
to mark certain locations. For different applications we might want the
markers to contain information. Along these lines, can you think of a way
you could extend this technique to use it in a game?

*/