/*

 ** IMPORTANT NOTE ** use map-3.dmi for this example.

Make sure that map-3.dmm is the only selected map. Both map-1.dmm and
map-2.dmm should be unchecked.


All of the previous examples had very few mobs. A common problem is when
games have large maps, they have lots of enemies. If all of the enemies
are running their AI routines, this can take up a lot of the CPU's processing
time.

In this example we use the mob AI from code-04.dm. This is very simple AI, but
if you have enough mobs it can still slow the game down.

We use a 400x400 map and place 3000 enemies on it.


  -- When you run this example --

Look at the statpanel to see what your CPU usage is. In code-19b.dm we'll look
at how to fix this. You'll need to see what your CPU usage is before the fix to
know what effect the fix has.

You can change the value of the variable ENEMY_COUNT to make it spawn a
different amount of enemies. If the CPU usage is too high with 3000 enemies,
create fewer. If the CPU usage is low (0-4) add more enemies.

*/


var
	const
		TEAM_1 = 1
		TEAM_2 = 2
		TEAM_3 = 3

		ENEMY_COUNT = 3000

world
	view = 10
	mob = /mob/player
	New()
		for(var/i = 1 to ENEMY_COUNT)
			while(1)
				var/x = rand(1,200)
				var/y = rand(1,200)

				if(x < 20 && y < 20) continue

				var/turf/t = locate(x,y,1)
				if(t.density) continue

				if(prob(50))
					new /mob/ai/blue(t)
				else
					new /mob/ai/red(t)
				break

turf
	wall
		opacity = 1

mob
	Click()
		if(get_dist(src, usr) <= 1)
			usr.attack(src)

	Stat()
		stat("CPU", world.cpu)

mob
	var
		team = 0
	proc
		ai()
		hit_by(mob/m)
		attack(mob/target)
			show_hit(target)
			target.hit_by(src)

	player
		Login()
			. = ..()
			team = TEAM_1
			icon_state = "green"

	ai
		var
			mob/target

		New()
			..()
			spawn(rand(10,20)) ai()

		ai()
			if(target)
				if(get_dist(src, target) > 1)
					step_to(src, target)
					sleep(5)
				else
					attack(target)
					sleep(10)
			else
				get_target()
				if(target)
					sleep(5)
				else
					step_rand(src)
					sleep(20)

			spawn() ai()

		hit_by(mob/m)
			if(m.team == team) return

			if(!target)
				target = m

		proc
			get_target()

		red
			team = TEAM_2
			icon_state = "red"

			get_target()
				for(var/mob/m in oview(2, src))
					if(m.team != team)
						target = m
						return

		blue
			team = TEAM_2
			icon_state = "blue"